/*
 * checkerberry test center - https://www.checkerberry.de
 *
 * Copyright (c) 2010 ConceptPeople consulting gmbh
 *
 * Lizenziert unter der EUPL, Version 1.1 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/software/page/eupl5
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.conceptpeople.checkerberry.web.example.model;

import de.conceptpeople.checkerberry.web.environment.ContextProvider;

/**
 * Model für die Homepage der Test-Web-Anwendung.
 * <p/>
 * In diesem Modell werden zwei verschiedene Ansätze dargestellt. Die Methode
 * {@link #submitUserDetails(String, String, String, String, String, boolean, boolean)}
 * nimmt alle Parameter des Formulars entgegen, tr ägt sie ein und schickt das
 * Formular ab. Dieses Vorgehen ist gerade bei kleinen Formularen mit
 * Pflichtfeldern sinnvoll. In dem konkreten Beispiel kommt man aber schon an
 * die Grenzen des lesbaren. Aus diesem Grund wird hier auch ein Builder-Pattern
 * z.B. {@link #enterFirstname(String)} verwendet. Das ist flexibler und besser
 * lesbar.
 * <p/>
 * Es hängt von der konkreten Anwendung ab, welches Vorgehen sinnvoll ist. Auch
 * eine Kombination beider Wege ist möglich.
 */
public class NewUserPage extends AbstractBasePage {

    /**
     * Erzeugt ein neues Model für die User-Anlegen-Seite.
     * 
     * @param contextProvider
     *            Provider für die Kontextinformationen.
     */
    public NewUserPage(ContextProvider contextProvider) {
        super(contextProvider, "newUser");
    }

    /**
     * Trägt die Werte in die entsprechenden Felder ein und speichert die
     * Informationen.
     * 
     * @param firstname
     *            Vorname des Benutzers.
     * @param surname
     *            Nachname des Benutzers.
     * @param note
     *            Information des Benutzers.
     * @param genderOption
     *            Geschlecht des Benutzers.
     * @param pet
     *            Haustier des Benutzers.
     * @param newsletter
     *            Newsletter-Indikator.
     * @param agb
     *            AGB-Indikator.
     */
    public void submitUserDetails(String firstname, String surname, String note, String genderOption,
            String pet, boolean newsletter, boolean agb) {

        createFirstnameField().type(firstname);
        createSurnameField().type(surname);
        createNoteField().type(note);
        createGenderComboBox().selectByOption(genderOption);
        createPetRadioButton(pet).click();

        if (newsletter) {
            createNewsletterCheckbox().check();
        }
        if (agb) {
            createAgbCheckbox().check();
        }
        createSubmitButton().clickAndWaitForPage();
    }

    /**
     * Trägt den Vor- und Nachnamen in das Formular ein.
     * 
     * @param firstname
     *            einzutragender Vorname.
     * @param lastname
     *            einzutragender Nachname.
     * @return aktuelles Modell zur weiteren Eingabe.
     */
    public NewUserPage enterName(String firstname, String lastname) {
        createFirstnameField().type(firstname);
        createSurnameField().type(lastname);
        return this;
    }

    /**
     * Trägt die Anmerkung in das Formular ein.
     * 
     * @param note
     *            einzutragende Anmerkung.
     * @return aktuelles Modell zur weiteren Eingabe.
     */
    public NewUserPage enterNote(String note) {
        createNoteField().type(note);
        return this;
    }

    /**
     * Wählt das Geschlecht in dem Formular aus. Hier wäre die Verwendung eines
     * Enums sinnvoller. Unter realen Bedingungen existiert in der Regel im
     * produktiven Code ein entsprechendes Enum, das in den Modellklassen
     * wiederverwendet werden kann. Das hat den Vorteil, dass Änderungen an den
     * Produktionsklasse, direkte Auswirkungen auf die Testklassen haben.
     * 
     * @param option
     *            ausgewähltes Geschlecht.
     * @return aktuelles Modell zur weiteren Eingabe.
     */
    public NewUserPage selectGender(String option) {
        createGenderComboBox().selectByOption(option);
        return this;
    }

    /**
     * Wählt das Haustier im Formular aus.
     * 
     * @param pet
     *            ausgewähltes Haustier.
     * @return aktuelles Modell zur weiteren Eingabe.
     * @see #selectGender(String)
     */
    public NewUserPage selectPet(String pet) {
        createPetRadioButton(pet).click();
        return this;
    }

    /**
     * Wählt die Checkbox zur Bestellung des Newsletters im Formular aus.
     * 
     * @return aktuelles Modell zur weiteren Eingabe.
     */
    public NewUserPage checkNewsletter() {
        createNewsletterCheckbox().check();
        return this;
    }

    /**
     * Wählt die Checkbox zur Bestätigung der AGB im Formular aus.
     * 
     * @return aktuelles Modell zur weiteren Eingabe.
     */
    public NewUserPage checkAgb() {
        createAgbCheckbox().check();
        return this;
    }

    /**
     * Schickt das aktuelle Formular ab.
     */
    public void clickAndWaitForPage() {
        createSubmitButton().clickAndWaitForPage();
    }

    /**
     * Erzeugt eine Fernsteuerung für die entsprechende Komponente.
     * 
     * @return erzeute Proxy-Komponente.
     */
    public BaseComponent createFirstnameField() {
        return createComponentProxy("firstname");
    }

    /**
     * Erzeugt eine Fernsteuerung für die entsprechende Komponente.
     * 
     * @return erzeute Proxy-Komponente.
     */
    public BaseComponent createSurnameField() {
        return createComponentProxy("surname");
    }

    /**
     * Erzeugt eine Fernsteuerung für die entsprechende Komponente.
     * 
     * @return erzeute Proxy-Komponente.
     */
    public BaseComponent createNoteField() {
        return createComponentProxy("note");
    }

    /**
     * Erzeugt eine Fernsteuerung für die entsprechende Komponente.
     * 
     * @return erzeute Proxy-Komponente.
     */
    public BaseComponent createGenderComboBox() {
        return createComponentProxy("gender");
    }

    /**
     * Erzeugt eine Fernsteuerung für die entsprechende Komponente.
     * 
     * @param value
     *            Value-Attribute des Radio-Buttons.
     * @return erzeute Proxy-Komponente.
     */
    public BaseComponent createPetRadioButton(String value) {
        return createComponentProxy("//input[@name='pet' and @value='" + value + "']");
    }

    /**
     * Erzeugt eine Fernsteuerung für die entsprechende Komponente.
     * 
     * @return erzeute Proxy-Komponente.
     */
    public BaseComponent createDogRadioButton() {
        return createPetRadioButton("dog");
    }

    /**
     * Erzeugt eine Fernsteuerung für die entsprechende Komponente.
     * 
     * @return erzeute Proxy-Komponente.
     */
    public BaseComponent createCatRadioButton() {
        return createPetRadioButton("cat");
    }

    /**
     * Erzeugt eine Fernsteuerung für die entsprechende Komponente.
     * 
     * @return erzeute Proxy-Komponente.
     */
    public BaseComponent createMouseRadioButton() {
        return createPetRadioButton("mouse");
    }

    /**
     * Erzeugt eine Fernsteuerung für die entsprechende Komponente.
     * 
     * @return erzeute Proxy-Komponente.
     */
    public BaseComponent createNewsletterCheckbox() {
        return createComponentProxy("newsletterOrdered");
    }

    /**
     * Erzeugt eine Fernsteuerung für die entsprechende Komponente.
     * 
     * @return erzeute Proxy-Komponente.
     */
    public BaseComponent createAgbCheckbox() {
        return createComponentProxy("agbAccepted");
    }

    /**
     * Erzeugt eine Fernsteuerung für die entsprechende Komponente.
     * 
     * @return erzeute Proxy-Komponente.
     */
    public BaseComponent createCountryComboBox() {
        return createComponentProxy("countryId");
    }

    /**
     * Erzeugt eine Fernsteuerung für die entsprechende Komponente.
     * 
     * @return erzeute Proxy-Komponente.
     */
    public BaseComponent createSubmitButton() {
        return createComponentProxy("sendId");
    }

}
