/*
 * checkerberry test center - https://www.checkerberry.de
 *
 * Copyright (c) 2010 ConceptPeople consulting gmbh
 *
 * Lizenziert unter der EUPL, Version 1.1 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/software/page/eupl5
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.conceptpeople.checkerberry.web.example.test;

import org.junit.Assert;
import org.junit.Test;

import com.thoughtworks.selenium.SeleniumException;

import de.conceptpeople.checkerberry.web.environment.TestUrl;
import de.conceptpeople.checkerberry.web.example.AbstractTestWebApplicationGuiTest;
import de.conceptpeople.checkerberry.web.example.model.BaseComponent;
import de.conceptpeople.checkerberry.web.example.model.LoginPage;
import de.conceptpeople.checkerberry.web.example.model.NewUserPage;
import de.conceptpeople.checkerberry.web.example.model.UserlistPage;
// CHECKSTYLE:OFF
import de.conceptpeople.checkerberry.web.remote.WebConfiguration;
// CHECKSTYLE:ON

/**
 * Abtrakter Test, der sowohl mit Selenium RC als auch mit WebDriver verwendet
 * wird. Über die Annotation {@link TestUrl} wird die zu testende URL definiert.
 * Diese wird bereits in der Setup-Phase geöffnet, sodass der Test direkt an der
 * Stelle starten kann.
 * <p/>
 * Die zu testende URL kann an drei Stellen definiert werden.
 * <ol>
 * <li>In der Konfiguration {@link WebConfiguration#getTestUrl()}: Diese Angabe
 * hat die niedrigste Priorität.</li>
 * <li>Über die Annotation {@link TestUrl} an der Klasse oder</li>
 * <li>Über die Annotation {@link TestUrl} an der Testmethode: Diese Angabe hat
 * die höchste Priorität.</li>
 * </ol>
 * <p/>
 * Anhand der Beispieltests können Sie sehen, wie flexibel Sie durch die
 * Verwendung eines Modells sind.
 */
@TestUrl("http://localhost:42001/")
public abstract class AbstractExampleGuiTest extends AbstractTestWebApplicationGuiTest {

    /**
     * Testet das Einloggen in die Testanwendung.
     */
    @Test
    public void testLogin() {

        LoginPage loginPage = new LoginPage(this);
        BaseComponent userField = loginPage.createUserField();
        BaseComponent passwordField = loginPage.createPasswordField();
        BaseComponent loginButton = loginPage.createLoginButton();

        // Sicherstellen, dass die Login-Seite angezeigt wird. Die Möglichkeit
        // jede Seite eindeutig zu identifizieren ist immer sinnvoll, da so auch
        // leicht die korrekte Navigation innerhalb der Anwendung getestet
        // werden kann.
        Assert.assertTrue(loginPage.isDisplayed());

        // User und Passwort sind leer -> Fehler...
        loginButton.clickAndWaitForPage();
        // ...und bleiben auf der aktuellen Seite.
        Assert.assertTrue(loginPage.isDisplayed());

        // User eintragen und Passwort leer lassen -> Fehler...
        userField.type("Hans");
        loginButton.clickAndWaitForPage();
        // ...und bleiben auf der aktuellen Seite.
        Assert.assertTrue(loginPage.isDisplayed());

        // Passwort hinzufügen...
        passwordField.type("Albers");
        // ...und Submit-Button drücken.
        loginButton.clickAndWaitForPage();

        // Jetzt hat das Login funktioniert...
        NewUserPage newUserPage = new NewUserPage(this);
        // ...und die Seite zur Eingabe der Benutzerinformationen wird
        // angezeigt.
        Assert.assertTrue(newUserPage.isDisplayed());
    }

    /**
     * Testet die Methode {@link BaseComponent#getAttribute(String)}. Es ist zu
     * beachten, dass die Implementierung der Fernsteuerung (Selenium RC oder
     * WebDriver) teilweise unterschiedliche Ergebnisse liefert. Über die
     * Konfigurationseinstellung
     * {@link WebConfiguration#setCompatibilityMode(boolean)} werden die
     * Abweichungen vereinheitlicht.
     */
    @Test
    public void testGetAttribute() {

        // Erzeugen des Modells für die Login-Seite.
        LoginPage loginPage = new LoginPage(this);
        BaseComponent userField = loginPage.createUserField();

        // Das Textfeld sollte vorhanden sein.
        Assert.assertTrue(userField.isAvailable());
        // Wenn das Attribut nicht vorhanden ist, kommt ein Leerstring zurück.
        Assert.assertEquals("", userField.getAttribute("not-there"));
        // Wenn das Attribute vorhanden ist, kommt der Wert zurück.
        Assert.assertEquals("text", userField.getAttribute("type"));

        // Erzeugen eines neuen Modells, obwohl diese Seite aktuell nicht
        // dargestellt wird.
        NewUserPage newUserPage = new NewUserPage(this);
        BaseComponent firstnameField = newUserPage.createFirstnameField();
        // Das Textfeld ist dementsprechend nicht vorhanden.
        Assert.assertFalse(firstnameField.isAvailable());
        try {
            // Die Abfrage eines Attributs führt in diesem Fall zu einer
            // Exception, da nicht nur das Attribut fehlt, sondern das gesamte
            // Feld nicht vorhanden ist.
            firstnameField.getAttribute("type");
            Assert.fail("Exception expected.");
        } catch (SeleniumException e) {
            // Das wurde erwartet.
        }
    }

    /**
     * Testet die unterschiedlichen Selektionsarten für ComboBoxen.
     */
    @Test
    public void testSelectBy() {

        // Standard-User anmelden...
        LoginPage loginPage = new LoginPage(this);
        Assert.assertTrue(loginPage.isDisplayed());
        loginPage.loginDefaultUser();

        // Auf der Seite zur Anlage eines neuen Benutzers werden die
        // verschiedenen Möglichkeiten getestet.
        NewUserPage newUserPage = new NewUserPage(this);
        Assert.assertTrue(newUserPage.isDisplayed());

        BaseComponent genderComboBox = newUserPage.createGenderComboBox();

        // "Männlich" über den Index auswählen. Das sollte nur gemacht werden,
        // wenn man wirklich sicher bzgl. der Indizes ist.
        genderComboBox.selectByIndex(1);
        Assert.assertEquals("m", genderComboBox.getValue());

        // "Weiblich" über das Label auswählen. Das ist in der Regel keine gute
        // Idee, da die Labels sich ändern können. Außerdem ist der Test dann an
        // eine Sprache gebunden. Man kann natürlich das gleiche Resource-Bundle
        // wie die Anwendung verwenden...
        genderComboBox.selectByLabel("weiblich");
        Assert.assertEquals("f", genderComboBox.getValue());

        // Die Auswahl über die Option ist in der Regel der robusteste Weg. Es
        // sollte dann jedoch auch ein Enum für die Option verwende werden.
        // Oftmals wird im produktiven Code bereits ein Enum verwendet, das im
        // Test wiederverwendet werden kann. Das ist die beste Möglichkeit, da
        // die Tests dann sehr eng an den produktiven Code gekoppelt sind.
        genderComboBox.selectByOption("-");
        Assert.assertEquals("-", genderComboBox.getValue());

        // Ähnliches noch einmal für die Länder.
        BaseComponent countryComboBox = newUserPage.createCountryComboBox();
        // Guter Zugriff s.o.
        countryComboBox.selectByOption("germany");
        Assert.assertEquals("germany", countryComboBox.getValue());
        // Auch eine sehr gute Möglichkeit für den Zugriff.
        countryComboBox.selectById("dk");
        Assert.assertEquals("danmark", countryComboBox.getValue());
    }

    /**
     * Testet die Anlage eines neuen Users.
     */
    @Test
    public void testNewUser() {

        // Einloggen.
        LoginPage loginPage = new LoginPage(this);
        Assert.assertTrue(loginPage.isDisplayed());
        loginPage.loginDefaultUser();

        // Seite für die Benutzereingabe.
        NewUserPage newUserPage = new NewUserPage(this);
        Assert.assertTrue(newUserPage.isDisplayed());

        BaseComponent firstnameField = newUserPage.createFirstnameField();

        // Warten bis das Feld in der Seite vorhanden ist. Diese Art zu warten
        // ist gerade bei Ajax-Aufrufen sehr effizient, da keine unnötigen
        // Wartezeiten verwendet werden. Zeit ist ein hohes Gut bei der
        // Durchführung von GUI-Tests!
        firstnameField.waitUntilAvailable();

        // Prüfen, ob das Feld in der Seite vorhanden ist. Beide Methoden machen
        // das gleiche...
        Assert.assertTrue(firstnameField.isElementPresent());
        Assert.assertTrue(firstnameField.isAvailable());

        // Eintragen der Werte über das Builder-Pattern.
        newUserPage.enterName("Homer", "Simpson").enterNote("Zusätzliche Information.");

        // Auswählen des Geschlechts über einen Index.
        BaseComponent genderComboBox = newUserPage.createGenderComboBox();
        genderComboBox.selectByIndex(1);

        // Auswählen des Haustiers direkt über den "Katzen-Button".
        newUserPage.createCatRadioButton().click();
        // Newsletter-Checkbox auswählen.
        newUserPage.createNewsletterCheckbox().check();

        // Button drücken und warten.
        newUserPage.createSubmitButton().clickAndWaitForPage();
    }

    /**
     * Testet die Aktualisierung eines Users.
     */
    @Test
    public void testUpdateUser() {

        // Einloggen.
        LoginPage loginPage = new LoginPage(this);
        Assert.assertTrue(loginPage.isDisplayed());
        loginPage.loginDefaultUser();

        // Seite für die Benutzereingabe.
        NewUserPage newUserPage = new NewUserPage(this);
        Assert.assertTrue(newUserPage.isDisplayed());

        // Die Simspons anlegen.
        newUserPage.submitUserDetails("Homer", "Simpson", "Dad", "m", "dog", true, false);
        newUserPage.submitUserDetails("Marge", "Simpson", "Mum", "f", "cat", false, true);
        newUserPage.submitUserDetails("Bart", "Simpson", "Son", "m", "dog", true, true);
        newUserPage.submitUserDetails("Lisa", "Simpson", "Brainy", "f", "cat", false, true);

        // Alternativ kann man auch das Builder-Pattern verwenden. Das ergibt
        // gerade bei größeren Formularen Sinn.
        newUserPage.enterName("Maggie", "Simpson").enterNote("Sweety").selectGender("f").selectPet("mouse")
                .clickAndWaitForPage();

        // Wechseln auf die Benutzerliste.
        newUserPage.getNavigation().createUserlistLink().clickAndWaitForPage();

        // Die Simpsons werden hier sortiert angezeigt:
        // 1. Header
        // 2. Bart
        // 3. Homer
        // 4. Lisa
        // 5. Maggie
        // 6. Marge

        UserlistPage userlistPage = new UserlistPage(this);
        Assert.assertTrue(userlistPage.isDisplayed());

        // Prüfen der Anzahl der Einträge.
        Assert.assertEquals(6, userlistPage.createUserlistTable().getRowCount());

        // Exemplarische Prüfung eines Eintrags. Die Prüfung könnte man auch in
        // eine eigene Methode assertRowContent(UserlistPage, ...) o.ä.
        // auslagern. Die Parametrisierung der Felder über die Zeilennummer ist
        // wieder ein großer Vorteil des Modellansatzes.
        int rowNumber = 4;
        Assert.assertEquals("Lisa", userlistPage.createFirstnameLabel(rowNumber).getText());
        Assert.assertEquals("Simpson", userlistPage.createSurnameLabel(rowNumber).getText());
        Assert.assertEquals("weiblich", userlistPage.createGenderLabel(rowNumber).getText());
        Assert.assertEquals("Nein", userlistPage.createNewsletterLabel(rowNumber).getText());

        // Lisa zur Bearbeitung auswählen.
        userlistPage.createEditButton(rowNumber).clickAndWaitForPage();

        // Prüfen der Einträge in der Benutzereingabe.
        BaseComponent firstnameField = newUserPage.createFirstnameField();
        BaseComponent surnameField = newUserPage.createSurnameField();
        BaseComponent noteField = newUserPage.createNoteField();
        BaseComponent genderComboBox = newUserPage.createGenderComboBox();

        Assert.assertEquals("Lisa", firstnameField.getValue());
        Assert.assertEquals("Simpson", surnameField.getValue());
        Assert.assertEquals("Brainy", noteField.getValue());
        Assert.assertEquals("f", genderComboBox.getValue());
        Assert.assertFalse(newUserPage.createDogRadioButton().isChecked());
        Assert.assertTrue(newUserPage.createCatRadioButton().isChecked());
        Assert.assertFalse(newUserPage.createMouseRadioButton().isChecked());
        Assert.assertFalse(newUserPage.createNewsletterCheckbox().isChecked());
        Assert.assertTrue(newUserPage.createAgbCheckbox().isChecked());

        // Prüfen der schreibgeschützen Felder.
        Assert.assertFalse(firstnameField.isEditable());
        Assert.assertTrue(firstnameField.isVisible());

        Assert.assertFalse(surnameField.isEditable());
        Assert.assertTrue(surnameField.isVisible());

        Assert.assertFalse(noteField.isEditable());
        Assert.assertFalse(noteField.isVisible());

        Assert.assertTrue(genderComboBox.isEditable());
        Assert.assertTrue(genderComboBox.isVisible());
    }

    /**
     * Testet die Funktionen der Checkbox.
     */
    @Test
    public void testCheckBoxes() {

        // Einloggen.
        LoginPage loginPage = new LoginPage(this);
        Assert.assertTrue(loginPage.isDisplayed());
        loginPage.loginDefaultUser();

        // Seite für die Benutzereingabe.
        NewUserPage newUserPage = new NewUserPage(this);
        Assert.assertTrue(newUserPage.isDisplayed());

        // Prüfen der Checkbox-Methoden.
        BaseComponent newsletterCheckbox = newUserPage.createNewsletterCheckbox();
        Assert.assertFalse(newsletterCheckbox.isChecked());

        newsletterCheckbox.check();
        Assert.assertTrue(newsletterCheckbox.isChecked());

        newsletterCheckbox.uncheck();
        Assert.assertFalse(newsletterCheckbox.isChecked());
    }

}
